@@ -35,15 +35,17 @@ typedef unsigned long long u64;
 #error I need at least some 64-bit type
 #endif
 
-#ifdef HAVE_LOCAL_NBD_H
+/* 2.6.18 and above use __be* rather than u* */
+#define __be32 u32
+#define __be64 u64
+
+#ifdef NBD_H_LOCAL
 #include "nbd.h"
 #else
-#ifdef HAVE_LINUX_NBD_H
+#ifdef NBD_H_LINUX
 #include <linux/nbd.h>
-#else
-#error I need an nbd.h somewhere. Either install it in <linux/nbd.h> or put it in the current directory.
-#endif
-#endif
+#endif // NBD_H_LINUX
+#endif // NBD_H_LOCAL
 
 #if NBD_LFS==1
 #define _LARGEFILE_SOURCE
@@ -55,7 +57,8 @@ u64 cliserv_magic = 0x00420281861253LL;
 
 #define INFO(a) do { } while(0)
 
-void setmysockopt(int sock) {
+void setmysockopt(int sock)
+{
 	int size = 1;
 #if 0
 	if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, &size, sizeof(int)) < 0)
@@ -73,20 +76,20 @@ void setmysockopt(int sock) {
 #endif
 }
 
-void err_noexit(const char *s) {
+void err(const char *s)
+{
 	const int maxlen = 150;
 	char s1[maxlen], *s2;
-	int n = 0;
 
 	strncpy(s1, s, maxlen);
-	if (s2 = strstr(s, "%m")) {
+	if ((s2 = strstr(s, "%m"))) {
 		strcpy(s1 + (s2 - s), strerror(errno));
 		s2 += 2;
 		strcpy(s1 + strlen(s1), s2);
 	}
 #ifndef	sun
 	/* Solaris doesn't have %h in syslog */
-	else if (s2 = strstr(s, "%h")) {
+	else if ((s2 = strstr(s, "%h"))) {
 		strcpy(s1 + (s2 - s), hstrerror(h_errno));
 		s2 += 2;
 		strcpy(s1 + strlen(s1), s2);
@@ -99,14 +102,11 @@ void err_noexit(const char *s) {
 #else
 	fprintf(stderr, "Error: %s\n", s1);
 #endif
-}
-
-void err(const char *s) {
-	err_noexit(s);
 	exit(1);
 }
 
-void logging(void) {
+void logging(void)
+{
 #ifdef ISSERVER
 	openlog(MY_NAME, LOG_PID, LOG_DAEMON);
 #endif
@@ -115,11 +115,13 @@ void logging(void) {
 }
 
 #ifdef WORDS_BIGENDIAN
-u64 ntohll(u64 a) {
+u64 ntohll(u64 a)
+{
 	return a;
 }
 #else
-u64 ntohll(u64 a) {
+u64 ntohll(u64 a)
+{
 	u32 lo = a & 0xffffffff;
 	u32 hi = a >> 32U;
 	lo = ntohl(lo);
